@@ -3,6 +3,7 @@ package Geo::Coder::Free;
 # TODO: Don't have Maxmind as a separate database
 # TODO: Rename openaddresses.sql as geo_coder_free.sql
 # TODO: Consider Data::Dumper::Names instead of Data::Dumper
+# TODO: use the cache to store common queries
 
 use strict;
 use warnings;
@@ -11,12 +12,14 @@ use warnings;
 
 use Array::Iterator;
 use Config::Auto;
+use Data::Dumper;
 use Geo::Coder::Abbreviations;
 use Geo::Coder::Free::MaxMind;
 use Geo::Coder::Free::OpenAddresses;
 use List::MoreUtils;
 use Locale::US;
 use Carp;
+use Scalar::Util;
 
 =head1 NAME
 
@@ -24,11 +27,11 @@ Geo::Coder::Free - Provides a Geo-Coding functionality using free databases
 
 =head1 VERSION
 
-Version 0.37
+Version 0.38
 
 =cut
 
-our $VERSION = '0.37';
+our $VERSION = '0.38';
 
 our $alternatives;
 our $abbreviations;
@@ -55,7 +58,9 @@ sub _normalize($);
 
 =head1 DESCRIPTION
 
-Geo::Coder::Free provides an interface to free databases by acting as a front-end to
+Geo::Coder::Free provides an interface
+to translate addresses into latitude and longitude
+by using to free databases such as
 L<Geo::Coder::Free::MaxMind> and L<Geo::Coder::Free::OpenAddresses>.
 
 The cgi-bin directory contains a simple DIY Geo-Coding website.
@@ -67,6 +72,8 @@ When it's back up you will be able to use this to test it.
 
     curl 'https://geocode.nigelhorne.com/cgi-bin/page.fcgi?page=query&q=1600+Pennsylvania+Avenue+NW+Washington+DC+USA'
 
+Includes functionality for running the module via the command line for testing or ad-hoc geocoding tasks.
+
 =head1 METHODS
 
 =head2 new
@@ -88,16 +95,28 @@ but that can't be guaranteed to work.
 
 sub new {
 	my $class = shift;
-	my %args = (ref($_[0]) eq 'HASH') ? %{$_[0]} : @_;
+
+	# Handle hash or hashref arguments
+	my %args;
+	if((@_ == 1) && (ref $_[0] eq 'HASH')) {
+		%args = %{$_[0]};
+	} elsif((@_ % 2) == 0) {
+		%args = @_;
+	} else {
+		carp(__PACKAGE__, ': Invalid arguments passed to new()');
+		return;
+	}
 
 	if(!defined($class)) {
-		# Using Geo::Coder::Free->new not Geo::Coder::Free::new
-		# carp(__PACKAGE__, ' use ->new() not ::new() to instantiate');
-		# return;
+		if((scalar keys %args) > 0) {
+			# Using Geo::Coder::Free->new not Geo::Coder::Free::new
+			carp(__PACKAGE__, ' use ->new() not ::new() to instantiate');
+			return;
+		}
 
 		# FIXME: this only works when no arguments are given
 		$class = __PACKAGE__;
-	} elsif(ref($class)) {
+	} elsif(Scalar::Util::blessed($class)) {
 		# clone the given object
 		return bless { %{$class}, %args }, ref($class);
 	}
@@ -119,7 +138,7 @@ sub new {
 		alternatives => $alternatives
 	};
 
-	if((!$args{'openaddr'}) && $ENV{'OPENADDR_HOME'}) {
+	if((!defined $args{'openaddr'}) && $ENV{'OPENADDR_HOME'}) {
 		$args{'openaddr'} = $ENV{'OPENADDR_HOME'};
 	}
 
@@ -147,14 +166,19 @@ sub new {
     # Note that this yields many false positives and isn't useable yet
     my @matches = $geo_coder->geocode(scantext => 'arbitrary text', region => 'US');
 
+    @matches = $geo_coder->geocode(scantext => 'arbitrary text', region => 'GB', ignore_words => [ 'foo', 'bar' ]);
+
 =cut
 
+# List of words that scantext should ignore
 my %common_words = (
 	'the' => 1,
 	'and' => 1,
 	'at' => 1,
 	'be' => 1,
 	'by' => 1,
+	'how' => 1,
+	'over' => 1,
 	'she' => 1,
 	'of' => 1,
 	'for' => 1,
@@ -162,6 +186,7 @@ my %common_words = (
 	'pm' => 1,
 	'in' => 1,
 	'an' => 1,
+	'more' => 1,
 	'to' => 1,
 	'road' => 1,
 	'is' => 1,
@@ -196,15 +221,50 @@ sub geocode {
 		$params{'location'} = shift;
 	}
 
+	# Fail when the input is just a set of numbers
+	if(defined($params{'location'}) && ($params{'location'} !~ /\D/)) {
+		Carp::croak('Usage: ', __PACKAGE__, ": invalid location to geocode(), $params{location}") if(length($params{'location'}));
+		return undef;
+	} elsif(defined($params{'scantext'}) && ($params{'scantext'} !~ /\D/)) {
+		Carp::croak('Usage: ', __PACKAGE__, ": invalid scantext to geocode(), $params{scantext}") if(length($params{'scantext'}));
+		return undef;
+	}
+
 	if($self->{'openaddr'}) {
 		if(wantarray) {
 			my @rc = $self->{'openaddr'}->geocode(\%params);
 			if((my $scantext = $params{'scantext'}) && (my $region = $params{'region'})) {
+				my %ignore_words;
+				if($params{'ignore_words'}) {
+					%ignore_words = map { lc($_) => 1 } @{$params{'ignore_words'}};
+				}
+				# ::diag(Data::Dumper->new([\%ignore_words])->Dump());
+				$region = uc($region);
+				if($region eq 'US') {
+					my @candidates = _find_us_addresses($scantext);
+					if(scalar(@candidates)) {
+						if(wantarray) {
+							my @us;
+							foreach my $candidate(@candidates) {
+								my @res = $self->{'maxmind'}->geocode("$candidate, USA");
+								push @us, @res;
+							}
+							return @us if(scalar(@us));
+						}
+						if(my $rc = $self->{'maxmind'}->geocode($candidates[0] . ', USA')) {
+							return $rc;
+						}
+					}
+				}
 				$scantext =~ s/[^\w']+/ /g;
 				my @a = List::MoreUtils::uniq(split(/\s/, $scantext));
 				my $iterator = Array::Iterator->new({ __array__ => \@a });
-				while(my $w = $iterator->get_next()) {
+				# TODO
+				# while(my $w = $iterator->get_next()) {
+				my $w;
+				if($w) {
 					next if(exists($common_words{lc($w)}));
+					next if(exists($ignore_words{lc($w)}));
 					if($w =~ /^[a-z]{2,}$/i) {
 						my $peek = $iterator->peek();
 						last if(!defined($peek));
@@ -229,22 +289,46 @@ sub geocode {
 				}
 
 				foreach my $word(List::MoreUtils::uniq(split(/\s/, $scantext))) {
+					# Ignore numbers
+					next if($word !~ /\D/);
 					# FIXME:  There are a *lot* of false positives
 					next if(exists($common_words{lc($word)}));
+					next if(exists($ignore_words{lc($word)}));
 					if($word =~ /^[a-z]{2,}$/i) {
 						my $key = "$word/$region";
 						my @matches;
-						if($self->{'scantext'}->{$key}) {
-							# ::diag("$key: HIT");
-							@matches = @{$self->{'scantext'}->{$key}};
+						if(my $hits = $self->{'scantext'}->{$key}) {
+							# ::diag("$key: HIT: ", Data::Dumper->new([$hits])->Dump());
+							@matches = @{$hits} if(scalar(@{$hits}));
 						} else {
 							# ::diag("$key: MISS");
 							@matches = $self->{'maxmind'}->geocode({ location => $word, region => $region });
 						}
-						$self->{'scantext'}->{$key} = \@matches;
-						@rc = (@rc, @matches);
+						# ::diag(__LINE__, Data::Dumper->new([\@matches])->Dump());
+						my @m;
+						foreach my $match(@matches) {
+							if(ref($match) eq 'HASH') {
+								$match->{'location'} = "$word, $region";
+								push @m, $match;
+							} elsif(ref($match) eq 'ARRAY') {
+								warn __PACKAGE__, ': TODO: handle array: ', Data::Dumper->new([$match])->Dump();
+							} else {
+								push @m, {
+									confidence => $match->confidence(),
+									location => $match->as_string(),
+									latitude => $match->lat(),
+									longitude => $match->long(),
+									lat => $match->lat(),
+									long => $match->long(),
+									database => $match->database(),
+									country => $match->country(),
+									city => $match->city()
+								}
+							}
+						}
+						$self->{'scantext'}->{$key} = \@m;
+						@rc = (@rc, @m);
 					}
-
 				}
 			}
 			return @rc if(scalar(@rc) && $rc[0]);
@@ -292,6 +376,34 @@ sub geocode {
 	}
 }
 
+# Function to find all possible US addresses in a string
+sub _find_us_addresses {
+	my ($text) = @_;
+	my @addresses;
+
+	# Regular expression to match U.S.-style addresses
+	my $address_regex = qr/
+		\b                 # Word boundary
+		(\d{1,5})          # Street number: 1 to 5 digits
+		\s+                # Space
+		([A-Za-z0-9\s]+?)  # Street name (alphanumeric, allows spaces)
+		\s+                # Space
+		(Street|St\.?|Avenue|Ave\.?|Boulevard|Blvd\.?|Road|Rd\.?|Lane|Ln\.?|Drive|Dr\.?) # Street type
+		(,\s+[A-Za-z\s]+)? # Optional city name
+		\s*                # Optional spaces
+		(,\s*[A-Z]{2})?    # Optional state abbreviation
+		\s*                # Optional spaces
+		(\d{5}(-\d{4})?)?  # Optional ZIP code
+		\b                 # Word boundary
+	/x;
+
+	# Find all matches
+	while ($text =~ /$address_regex/g) {
+		push @addresses, $&;	# Capture the full match
+	}
+
+	return @addresses;
+}
 =head2 reverse_geocode
 
     $location = $geocoder->reverse_geocode(latlng => '37.778907,-122.39732');
@@ -355,7 +467,8 @@ Does nothing, here for compatibility with other Geo-Coders
 
 =cut
 
-sub ua {
+sub ua
+{
 }
 
 =head2 run
@@ -449,27 +562,44 @@ This will take a long time and use a lot of disc space, be clear that this is wh
 In the bin directory there are some helper scripts to do this.
 You will need to tailor them to your set up, but that's not that hard as the
 scripts are trivial.
-1. mkdir $WHOSONFIRST_HOME, cd $WHOSONFIRST_HOME, run wof-clone from NJH-Snippets.
+
+=over 4
+
+=item 1
+
+C<mkdir $WHOSONFIRST_HOME; cd $WHOSONFIRST_HOME> run wof-clone from NJH-Snippets.
+
 This can take a long time because it contains lots of directories which filesystem drivers
 seem to take a long time to navigate (at least my EXT4 and ZFS systems do).
-2. Install L<https://github.com/dr5hn/countries-states-cities-database.git> into $DR5HN_HOME.
+
+=item 2
+
+Install L<https://github.com/dr5hn/countries-states-cities-database.git> into $DR5HN_HOME.
 This data contains cities only,
 so it's not used if OSM_HOME is set,
 since the latter is much more comprehensive.
 Also, only Australia, Canada and the US is imported, as the UK data is difficult to parse.
-3. Run bin/download_databases - this will download the WhosOnFirst, Openaddr,
+
+=item 3
+
+Run bin/download_databases - this will download the WhosOnFirst, Openaddr,
 Open Street Map and dr5hn databases.
 Check the values of OSM_HOME, OPENADDR_HOME,
 DR5HN_HOME and WHOSONFIRST_HOME within that script,
 you may wish to change them.
 The Makefile.PL file will download the MaxMind database for you, as that is not optional.
-4. Run bin/create_db - this creates the database used by G:C:F using the data you've just downloaded
+
+=item 4
+
+Run bin/create_db - this creates the database used by G:C:F using the data you've just downloaded
 The database is called openaddr.sql,
 even though it does include all of the above data.
 That's historical before I added the WhosOnFirst database.
 The names are a bit of a mess because of that.
 I should rename it to geo-coder-free.sql, even though it doesn't contain the Maxmind data.
 
+=back
+
 Now you're ready to run "make"
 (note that the download_databases script may have done that for you,
 but you'll want to check).
@@ -487,7 +617,7 @@ Unfortunately all of the on-line services have one problem or another - most eit
 Even my modest tree, just over 2000 people, reaches those limits.
 
 There are, however, a number of free databases that can be used, including MaxMind, GeoNames, OpenAddresses and WhosOnFirst.
-The objective of Geo::Coder::Free (L<https://github.com/nigelhorne/Geo-Coder-Free>)
+The objective of L<Geo::Coder::Free> (L<https://github.com/nigelhorne/Geo-Coder-Free>)
 is to create a database of those databases and to create a search engine either through a local copy of the database or through an on-line website.
 Both are in their early days, but I have examples which do surprisingly well.
 
@@ -512,6 +642,8 @@ The OpenAddresses data doesn't cover the globe.
 
 Can't parse and handle "London, England".
 
+It would be great to have a set-up wizard to create the database.
+
 The various scripts in NJH-Snippets ought to be in this module.
 
 =head1 SEE ALSO
@@ -575,7 +707,7 @@ L<http://search.cpan.org/dist/Geo-Coder-Free/>
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright 2017-2024 Nigel Horne.
+Copyright 2017-2025 Nigel Horne.
 
 The program code is released under the following licence: GPL for personal use on a single computer.
 All other users (including Commercial, Charity, Educational, Government)
