@@ -5,26 +5,30 @@ package Geo::Coder::Free::Display;
 
 =head1 VERSION
 
-Version 0.37
+Version 0.38
 
 =cut
 
-our $VERSION = '0.37';
+our $VERSION = '0.38';
 
 use strict;
 use warnings;
+
 use Config::Auto;
 use CGI::Info;
 use Data::Dumper;
 use File::Spec;
 use Template::Filters;
 use Template::Plugin::EnvHash;
+use Template::Plugin::Math;
 use HTML::SocialMedia;
 use Geo::Coder::Free::Utils;
 use Error;
 use Fatal qw(:void open);
 use File::pfopen;
+use Scalar::Util;
 
+# TODO: read this from the config file
 my %blacklist = (
 	'MD' => 1,
 	'RU' => 1,
@@ -47,10 +51,22 @@ our $sm;
 our $smcache;
 
 sub new {
-	my $proto = shift;
+	my $class = shift;
+
+	# Handle hash or hashref arguments
 	my %args = (ref($_[0]) eq 'HASH') ? %{$_[0]} : @_;
 
-	my $class = ref($proto) || $proto;
+	if(!defined($class)) {
+		# Using Geo::Coder::Free::Display->new(), not Geo::Coder::Free::Display::new()
+		# carp(__PACKAGE__, ' use ->new() not ::new() to instantiate');
+		# return;
+
+		# FIXME: this only works when no arguments are given
+		$class = __PACKAGE__;
+	} elsif(Scalar::Util::blessed($class)) {
+		# If $class is an object, clone it with new arguments
+		return bless { %{$class}, %args }, ref($class);
+	}
 
 	if(defined($ENV{'HTTP_REFERER'})) {
 		# Protect against Shellshocker
@@ -79,7 +95,7 @@ sub new {
 		Data::Throttler->import();
 
 		# Handle YAML Errors
-		my $db_file = $info->tmpdir() . '/throttle';
+		my $db_file = File::Spec->catdir($info->tmpdir(), 'throttle');
 		eval {
 			my $throttler = Data::Throttler->new(
 				max_items => 30,
@@ -91,6 +107,7 @@ sub new {
 			);
 
 			unless($throttler->try_push(key => $ENV{'REMOTE_ADDR'})) {
+				sleep(1);
 				die "$ENV{REMOTE_ADDR} connexion throttled";
 			}
 		};
@@ -103,52 +120,7 @@ sub new {
 			}
 		}
 	}
-	my $config_dir;
-	if($ENV{'CONFIG_DIR'}) {
-		$config_dir = $ENV{'CONFIG_DIR'};
-	} else {
-		$config_dir = File::Spec->catdir(
-				$info->script_dir(),
-				File::Spec->updir(),
-				File::Spec->updir(),
-				'conf'
-			);
-
-		if(!-d $config_dir) {
-			$config_dir = File::Spec->catdir(
-					$info->script_dir(),
-					File::Spec->updir(),
-					'conf'
-				);
-		}
-
-		if(!-d $config_dir) {
-			if($ENV{'DOCUMENT_ROOT'}) {
-				$config_dir = File::Spec->catdir(
-					$ENV{'DOCUMENT_ROOT'},
-					File::Spec->updir(),
-					'lib',
-					'conf'
-				);
-			} else {
-				$config_dir = File::Spec->catdir(
-					$ENV{'HOME'},
-					'lib',
-					'conf'
-				);
-			}
-		}
-
-		if(!-d $config_dir) {
-			if($args{config_directory}) {
-				$config_dir = $args{config_directory};
-			} elsif($args{logger}) {
-				while(my ($k, $v) = each %ENV) {
-					$args{logger}->debug("$k=$v");
-				}
-			}
-		}
-	}
+	my $config_dir = _find_config_dir(\%args, $info);
 	if($args{'logger'}) {
 		$args{'logger'}->debug(__PACKAGE__, ': ', __LINE__, " path = $config_dir");
 	}
@@ -203,9 +175,66 @@ sub new {
 	$self->{'_social_media'}->{'facebook_share_button'} = $sm->as_string(facebook_share_button => 1);
 	# $self->{'_social_media'}->{'google_plusone'} = $sm->as_string(google_plusone => 1);
 
+	# Return the blessed object
 	return bless $self, $class;
 }
 
+# Determine the configuration directory
+sub _find_config_dir
+{
+	my($args, $info) = @_;
+
+	if($ENV{'CONFIG_DIR'}) {
+		return $ENV{'CONFIG_DIR'};
+	}
+
+	my $config_dir = File::Spec->catdir(
+			$info->script_dir(),
+			File::Spec->updir(),
+			File::Spec->updir(),
+			'conf'
+		);
+
+	if(!-d $config_dir) {
+		$config_dir = File::Spec->catdir(
+				$info->script_dir(),
+				File::Spec->updir(),
+				'conf'
+			);
+	}
+
+	if(!-d $config_dir) {
+		if($ENV{'DOCUMENT_ROOT'}) {
+			$config_dir = File::Spec->catdir(
+				# $ENV{'DOCUMENT_ROOT'},
+				$info->rootdir(),
+				File::Spec->updir(),
+				'lib',
+				'conf'
+			);
+		} else {
+			$config_dir = File::Spec->catdir(
+				$ENV{'HOME'},
+				'lib',
+				'conf'
+			);
+		}
+	}
+
+	if(!-d $config_dir) {
+		if($args->{config_directory}) {
+			return $args->{config_directory};
+		}
+		if($args->{logger}) {
+			while(my ($k, $v) = each %ENV) {
+				$args->{logger}->debug("$k=$v");
+			}
+		}
+	}
+
+	return $config_dir;
+}
+
 # Call this to display the page
 # It calls http() to create the HTTP headers, then html() to create the body
 sub as_string {
@@ -214,12 +243,13 @@ sub as_string {
 	# TODO: Get all cookies and send them to the template.
 	# 'cart' is an example
 	unless($args && $args->{cart}) {
-		my $purchases = $self->{_info}->get_cookie(cookie_name => 'cart');
-		if($purchases) {
+		if(my $purchases = $self->{_info}->get_cookie(cookie_name => 'cart')) {
 			my %cart = split(/:/, $purchases);
 			$args->{cart} = \%cart;
 		}
 	}
+
+	# Calculate items in cart if not already present in $args
 	unless($args && $args->{itemsincart}) {
 		if($args->{cart}) {
 			my $itemsincart;
@@ -239,14 +269,15 @@ sub as_string {
 		# return;
 	# }
 	# return $self->http() . $html;
+
+	# Build the HTTP response
 	my $rc = $self->http();
-	if($rc =~ /^Location:\s/ms) {
-		return $rc;
-	}
-	return $rc . $self->html($args);
+	return $rc =~ /^Location:\s/ms ? $rc : $rc . $self->html($args);
 }
 
-sub get_template_path {
+# Determine the path to the correct template file based on various criteria such as language settings, browser type, and module path
+sub get_template_path
+{
 	my $self = shift;
 	my %args = (ref($_[0]) eq 'HASH') ? %{$_[0]} : @_;
 
@@ -265,39 +296,28 @@ sub get_template_path {
 	}
 	$dir .= '/templates';
 
+	my $prefix;
+
 	# Look in .../robot or .../mobile first, if appropriate
-	my $prefix = '';
-
 	# Look in .../en/gb/web, then .../en/web then /web
-	if($self->{_lingua}) {
-		my $lingua = $self->{_lingua};
-
-		$self->_debug({ message => 'Requested language: ' . $lingua->requested_language() });
-
-		# FIXME: look for lower priority languages if the highest isn't found
-		my $candidate;
-		if($lingua->sublanguage_code_alpha2()) {
-			$candidate = "$dir/" . $lingua->code_alpha2() . '/' . $lingua->sublanguage_code_alpha2();
-			$self->_debug({ message => "check for directory $candidate" });
-			if(!-d $candidate) {
-				$candidate = undef;
+	foreach my $browser_type($self->_types()) {
+		if(my $lingua = $self->{_lingua}) {
+			$self->_debug({ message => 'Requested language: ' . $lingua->requested_language() });
+			# FIXME: look for lower priority languages if the highest isn't found
+			if(my $language = $lingua->language_code_alpha2()) {
+				if(my $dialect = $lingua->sublanguage_code_alpha2()) {
+					$prefix .= "$dir/$browser_type/$language/$dialect:";
+					$prefix .= "$dir/$browser_type/$language/default:";
+				}
+				$prefix .= "$dir/$language/$browser_type:" if(-d "$dir/$language/$browser_type");
+				$prefix .= "$dir/$browser_type/$language:" if(-d "$dir/$browser_type/$language");
+				$prefix .= "$dir/$browser_type/default:" if(-d "$dir/$browser_type/default");
+				$prefix .= "$dir/default/$browser_type/:" if(-d "$dir/default/$browser_type");
 			}
 		}
-		if((!defined($candidate)) && defined($lingua->code_alpha2())) {
-			$candidate = "$dir/" . $lingua->code_alpha2();
-			$self->_debug({ message => "check for directory $candidate" });
-			if(!-d $candidate) {
-				$candidate = undef;
-			}
-		}
-		if($candidate) {
-			$prefix = $self->_append_browser_type({ directory => $candidate });
-		}
+		$prefix .= "$dir/$browser_type:" if(-d "$dir/$browser_type");
 	}
 
-	$prefix .= $self->_append_browser_type({ directory => "$dir/default" });
-	$prefix .= $self->_append_browser_type({ directory => $dir });
-
 	# Fall back to .../web, or if that fails, assume no web, robot or
 	# mobile variant
 	$prefix .= "$dir/web:$dir/default/web:$dir/default:$dir";
@@ -321,6 +341,12 @@ sub get_template_path {
 	close($fh);
 	$self->_debug({ message => "using $filename" });
 	$self->{_filename} = $filename;
+
+	# Remember the template filename
+	if($self->{'log'}) {
+		$self->{'log'}->template($filename);
+	}
+
 	return $filename;
 }
 
@@ -335,52 +361,45 @@ sub set_cookie
 	return $self;
 }
 
-sub http {
+sub http
+{
 	my $self = shift;
 
+	# Handle session cookies
 	# TODO: Only session cookies as the moment
-	my $cookies = $self->{_cookies};
-	if(defined($cookies)) {
+	if(my $cookies = $self->{_cookies}) {
 		foreach my $cookie (keys(%{$cookies})) {
-			if(exists($cookies->{$cookie})) {
-				print "Set-Cookie:$cookie=$cookies->{$cookie}; path=/; HttpOnly\n";
-			} else {
-				print "Set-Cookie:$cookie=0:0; path=/; HttpOnly\n";
-			}
+			my $value = exists $cookies->{$cookie} ? $cookies->{$cookie} : '0:0';
+			print "Set-Cookie: $cookie=$value; path=/; HttpOnly\n";
 		}
 	}
 
-	my $language;
-	if($self->{_lingua}) {
-		$language = $self->{_lingua}->language();
-	} else {
-		$language = 'English';
-	}
-
-	my $rc;
+	# Determine language, defaulting to English
+	# TODO: Change the headers, e.g. character set, based on the language
+	# my $language = $self->{_lingua} ? $self->{_lingua}->language() : 'English';
 
+	# Determine content type
 	my $filename = $self->get_template_path();
-	if($filename =~ /\.txt$/) {
+	my $rc;
+	if ($filename =~ /\.txt$/) {
 		$rc = "Content-Type: text/plain\n";
-	} elsif($language eq 'Japanese') {
-		binmode(STDOUT, ':utf8');
-
-		$rc = "Content-Type: text/html; charset=UTF-8\n";
-	} elsif($language eq 'Polish') {
-		binmode(STDOUT, ':utf8');
-
-		# print "Content-Type: text/html; charset=ISO-8859-2\n";
-		$rc = "Content-Type: text/html; charset=UTF-8\n";
 	} else {
-		# $rc = "Content-Type: text/html; charset=ISO-8859-1\n";
+		binmode(STDOUT, ':utf8');
 		$rc = "Content-Type: text/html; charset=UTF-8\n";
 	}
 
+	# Security headers
 	# https://www.owasp.org/index.php/Clickjacking_Defense_Cheat_Sheet
 	# https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/X-Content-Type-Options
-	return $rc . "X-Frame-Options: SAMEORIGIN\nX-Content-Type-Options: nosniff\nReferrer-Policy: strict-origin-when-cross-origin\n\n";
+
+	# TODO: investigate Content-Security-Policy
+	return $rc . "X-Frame-Options: SAMEORIGIN\n"
+		. "X-Content-Type-Options: nosniff\n"
+		. "Referrer-Policy: strict-origin-when-cross-origin\n\n";
 }
 
+# Run the given data through the template to create HTML
+
 # Override this routine in a subclass if you wish to create special arguments to
 # send to the template
 sub html {
@@ -389,6 +408,8 @@ sub html {
 
 	my $filename = $self->get_template_path();
 	my $rc;
+
+	# Handle template files (.tmpl or .t)
 	if($filename =~ /.+\.t(mpl|t)$/) {
 		require Template;
 		Template->import();
@@ -426,6 +447,7 @@ sub html {
 			ABSOLUTE => 1,
 		});
 
+		# Process the template
 		if(!$template->process($filename, $vals, \$rc)) {
 			if(my $err = $template->error()) {
 				throw Error::Simple($err);
@@ -433,6 +455,7 @@ sub html {
 			throw Error::Simple("Unknown error in template: $filename");
 		}
 	} elsif($filename =~ /\.(html?|txt)$/) {
+		# Handle static HTML or text files
 		open(my $fin, '<', $filename) || throw Error::Simple("$filename: $!");
 
 		my @lines = <$fin>;
@@ -444,8 +467,9 @@ sub html {
 		throw Error::Simple("Unhandled file type $filename");
 	}
 
-	if(($filename !~ /.txt$/) && ($rc =~ /\smailto:(.+?)>/) && ($1 !~ /^&/)) {
-		$self->_debug({ message => "Found mailto link $1, you should remove it or use " . obfuscate($1) . ' instead' });
+	# Check for mailto links and log a warning
+	if(($filename !~ /.txt$/) && ($rc =~ /\smailto:(.+?)>/) && ($1 !~ /^&/) && $self->{_logger}) {
+		$self->{_logger}->warn({ message => "Found mailto link $1, you should remove it or use " . obfuscate($1) . ' instead' });
 	}
 
 	return $rc;
@@ -455,12 +479,12 @@ sub _debug
 {
 	my $self = shift;
 
-	if($self->{_logger}) {
+	if(my $logger = $self->{_logger}) {
 		my %params = (ref($_[0]) eq 'HASH') ? %{$_[0]} : @_;
-		if($ENV{'REMOTE_ADDR'}) {
-			$self->{_logger}->debug("$ENV{'REMOTE_ADDR'}: $params{'message'}");
+		if(defined($ENV{'REMOTE_ADDR'})) {
+			$logger->debug("$ENV{'REMOTE_ADDR'}: $params{'message'}");
 		} else {
-			$self->{_logger}->debug($params{'message'});
+			$logger->debug($params{'message'});
 		}
 	}
 	return $self;
@@ -470,38 +494,22 @@ sub obfuscate {
 	return map { '&#' . ord($_) . ';' } split(//, shift);
 }
 
-sub _append_browser_type {
+sub _types
+{
 	my $self = shift;
-	my %args = (ref($_[0]) eq 'HASH') ? %{$_[0]} : @_;
+	my $info = $self->{_info};
+	my @rc;
 
-	if($self->{_logger}) {
-		$self->{_logger}->trace('_append_browser_type');
+	if($info->is_search_engine()) {
+		push @rc, 'search', 'robot';
+	} elsif($info->is_mobile()) {
+		push @rc, 'mobile';
+	} elsif($info->is_robot()) {
+		push @rc, 'robot', 'search';
 	}
+	push @rc, 'web';
 
-	my $directory = $args{'directory'};
-
-	return unless(defined($directory));
-
-	if($self->{_logger}) {
-		$self->{_logger}->debug("_append_browser_type: directory = $directory");
-	}
-
-	my $rc;
-	if(-d $directory) {
-		if($self->{_info}->is_search_engine()) {
-			$rc = "$directory/search:$directory/robot:";
-		} elsif($self->{_info}->is_mobile()) {
-			$rc = "$directory/mobile:";
-		} elsif($self->{_info}->is_robot()) {
-			$rc = "$directory/robot:$directory/search:";
-		}
-		$rc .= "$directory/web:";
-
-		$self->_debug({ message => "_append_directory_type: $directory=>$rc" });
-		return $rc;
-	}
-
-	return '';	# Don't return undef or else the caller may use an uninit variable
+	return @rc;
 }
 
 1;
