@@ -5,11 +5,11 @@ package main;
 
 =head1 VERSION
 
-Version 0.37
+Version 0.38
 
 =cut
 
-our $VERSION = '0.37';
+our $VERSION = '0.38';
 
 use strict;
 use warnings;
@@ -24,33 +24,42 @@ BEGIN {
 	Log::Any::Adapter->set('Log4perl');
 }
 
-sub create_disc_cache {
-	my %args = (ref($_[0]) eq 'HASH') ? %{$_[0]} : @_;
+=head1 SUBROUTINES/METHODS
 
-	my $config = $args{'config'};
+=head2 create_disc_cache
+
+Initialise a disc-based cache using the CHI module.
+Supports multiple cache drivers, including BerkeleyDB, DBI, and Redis.
+
+=cut
+
+sub create_disc_cache {
+	my $args = get_params(undef, @_);
+
+	my $config = $args->{'config'};
 	throw Error::Simple('config is not optional') unless($config);
 
-	my $logger = $args{'logger'};
+	my $logger = $args->{'logger'};
 	my $driver = $config->{disc_cache}->{driver};
 	unless(defined($driver)) {
-		my $root_dir = $ENV{'root_dir'} || $args{'root_dir'} || $config->{disc_cache}->{root_dir} || $config->{'root_dir'};
+		my $root_dir = $ENV{'root_dir'} || $args->{'root_dir'} || $config->{disc_cache}->{root_dir} || $config->{'root_dir'};
 		throw Error::Simple('root_dir is not optional') unless($root_dir);
 
 		if($logger) {
 			$logger->warn(Data::Dumper->new([$config])->Dump());
 			$logger->warn('disc_cache not defined in ', $config->{'config_path'}, ' falling back to BerkeleyDB');
 		}
-		return CHI->new(driver => 'BerkeleyDB', root_dir => $root_dir, namespace => $args{'namespace'});
+		return CHI->new(driver => 'BerkeleyDB', root_dir => $root_dir, namespace => $args->{'namespace'});
 	}
 	if($logger) {
-		$logger->debug('disc cache via ', $config->{disc_cache}->{driver}, ', namespace: ', $args{'namespace'});
+		$logger->debug('disc cache via ', $config->{disc_cache}->{driver}, ', namespace: ', $args->{'namespace'});
 	}
 
 	my %chi_args = (
 		on_get_error => 'warn',
 		on_set_error => 'die',
 		driver => $driver,
-		namespace => $args{'namespace'}
+		namespace => $args->{'namespace'}
 	);
 
 	# Don't do this because it takes a lot of complex configuration
@@ -93,7 +102,7 @@ sub create_disc_cache {
 		);
 		$chi_args{'redis_options'} = \%redis_options;
 	} elsif($driver ne 'Null') {
-		$chi_args{'root_dir'} = $ENV{'root_dir'} || $args{'root_dir'} || $config->{disc_cache}->{root_dir};
+		$chi_args{'root_dir'} = $ENV{'root_dir'} || $args->{'root_dir'} || $config->{disc_cache}->{root_dir};
 		throw Error::Simple('root_dir is not optional') unless($chi_args{'root_dir'});
 		if($logger) {
 			$logger->debug("root_dir: $chi_args{root_dir}");
@@ -102,31 +111,38 @@ sub create_disc_cache {
 	return CHI->new(%chi_args);
 }
 
-sub create_memory_cache {
-	my %args = (ref($_[0]) eq 'HASH') ? %{$_[0]} : @_;
+=head2 create_memory_cache
 
-	my $config = $args{'config'};
+Initialise a memory-based cache using the CHI module.
+Supports multiple cache drivers, including SharedMem, Memory, and Redis.
+
+=cut
+
+sub create_memory_cache {
+	my $args = get_params(undef, @_);
+
+	my $config = $args->{'config'};
 	throw Error::Simple('config is not optional') unless($config);
 
-	my $logger = $args{'logger'};
+	my $logger = $args->{'logger'};
 	my $driver = $config->{memory_cache}->{driver};
 	unless(defined($driver)) {
 		if($logger) {
 			$logger->warn('memory_cache not defined in ', $config->{'config_path'}, ' falling back to sharedmem');
 		}
-		# return CHI->new(driver => 'Memcached', servers => [ '127.0.0.1:11211' ], namespace => $args{'namespace'});
-		# return CHI->new(driver => 'File', root_dir => '/tmp/cache', namespace => $args{'namespace'});
-		return CHI->new(driver => 'SharedMem', max_size => 1024, shm_size => 16 * 1024, shm_key => 98766789, namespace => $args{'namespace'});
+		# return CHI->new(driver => 'Memcached', servers => [ '127.0.0.1:11211' ], namespace => $args->{'namespace'});
+		# return CHI->new(driver => 'File', root_dir => '/tmp/cache', namespace => $args->{'namespace'});
+		return CHI->new(driver => 'SharedMem', max_size => 1024, shm_size => 16 * 1024, shm_key => 98766789, namespace => $args->{'namespace'});
 }
 	if($logger) {
-		$logger->debug('memory cache via ', $config->{memory_cache}->{driver}, ', namespace: ', $args{'namespace'});
+		$logger->debug('memory cache via ', $config->{memory_cache}->{driver}, ', namespace: ', $args->{'namespace'});
 	}
 
 	my %chi_args = (
 		on_get_error => 'warn',
 		on_set_error => 'die',
 		driver => $driver,
-		namespace => $args{'namespace'}
+		namespace => $args->{'namespace'}
 	);
 
 	if($logger) {
@@ -152,15 +168,15 @@ sub create_memory_cache {
 		}
 		$chi_args{'servers'} = \@servers;
 	} elsif($driver eq 'SharedMem') {
-		$chi_args{'shm_key'} = $args{'shm_key'} || $config->{memory_cache}->{shm_key};
-		if(my $shm_size = ($args{'shm_size'} || $config->{'memory_cache'}->{'shm_size'})) {
+		$chi_args{'shm_key'} = $args->{'shm_key'} || $config->{memory_cache}->{shm_key};
+		if(my $shm_size = ($args->{'shm_size'} || $config->{'memory_cache'}->{'shm_size'})) {
 			$chi_args{'shm_size'} = $shm_size;
 		}
-		if(my $max_size = ($args{'max_size'} || $config->{'memory_cache'}->{'max_size'})) {
+		if(my $max_size = ($args->{'max_size'} || $config->{'memory_cache'}->{'max_size'})) {
 			$chi_args{'max_size'} = $max_size;
 		}
 	} elsif(($driver ne 'Null') && ($driver ne 'Memory')) {
-		$chi_args{'root_dir'} = $ENV{'root_dir'} || $args{'root_dir'} || $config->{memory_cache}->{root_dir} || $config->{'root_dir'};
+		$chi_args{'root_dir'} = $ENV{'root_dir'} || $args->{'root_dir'} || $config->{memory_cache}->{root_dir} || $config->{'root_dir'};
 		throw Error::Simple('root_dir is not optional') unless($chi_args{'root_dir'});
 		if($logger) {
 			$logger->debug("root_dir: $chi_args{root_dir}");
@@ -175,8 +191,49 @@ sub create_memory_cache {
 	return CHI->new(%chi_args);
 }
 
-# From http://www.geodatasource.com/developers/perl
-# FIXME:  use Math::Trig
+=head2	get_params
+
+Parse the arguments given to a function,
+allowing the caller to call the function in anyway that they want e.g. foo('bar'), foo(arg => 'bar'), foo({ arg => 'bar' })
+all mean the same when called _get_params('arg', @_);
+
+=cut
+
+sub get_params
+{
+	my $default = shift;
+
+	# Directly return hash reference if the first parameter is a hash reference
+	return $_[0] if ref $_[0] eq 'HASH';
+
+	my %rc;
+	my $num_args = scalar @_;
+
+	# Populate %rc based on the number and type of arguments
+	if(($num_args == 1) && (defined $default)) {
+		# %rc = ($default => shift);
+		return { $default => shift };
+	} elsif($num_args == 1) {
+		throw Error::Simple('Usage: ' . __PACKAGE__ . '->' . (caller(1))[3] . '()');
+	} elsif($num_args == 0 && defined $default) {
+		throw Error::Simple('Usage: ' . __PACKAGE__ . '->' . (caller(1))[3] . '($default => \$val)');
+	} elsif(($num_args % 2) == 0) {
+		%rc = @_;
+	}
+
+	return \%rc;
+}
+
+=head2 distance
+
+Calculate the distance between two geographical points using latitude and longitude.
+Supports distance in kilometres (K), nautical miles (N), or miles.
+
+From L<http://www.geodatasource.com/developers/perl>
+FIXME:  use Math::Trig
+
+=cut
+
 sub distance {
 	my ($lat1, $lon1, $lat2, $lon2, $unit) = @_;
 	my $theta = $lon1 - $lon2;
@@ -184,9 +241,9 @@ sub distance {
 	$dist = _acos($dist);
 	$dist = _rad2deg($dist);
 	$dist = $dist * 60 * 1.1515;
-	if ($unit eq "K") {
-		$dist = $dist * 1.609344;
-	} elsif ($unit eq "N") {
+	if ($unit eq 'K') {
+		$dist = $dist * 1.609344;	# number of kilometres in a mile
+	} elsif ($unit eq 'N') {
 		$dist = $dist * 0.8684;
 	}
 	return ($dist);
